from typing import List, Dict
import statistics


class Results:
    """
    Results handles calculating statistics based on a list of requests that were made.
    Here's an example of what the information will look like:

    Successful requests     3000
    Slowest                 0.010s
    Fastest                 0.001s
    Average                 0.003s
    Total time              2.400s
    Requests Per Minute     90000
    Requests Per Second     125
    """

    def __init__(self, total_time: float, requests: List[Dict]):
        self.total_time = total_time
        self.requests = sorted(requests, key=lambda r: r["request_time"])

    def slowest(self) -> float:
        """
        Returns the slowest request's completion time

        >>> results = Results(10.6, [{
        ...     'status_code': 200,
        ...     'request_time': 3.4
        ... }, {
        ...     'status_code': 500,
        ...     'request_time': 6.1
        ... }, {
        ...     'status_code': 200,
        ...     'request_time': 1.04
        ... }])
        >>> results.slowest()
        6.1
        """
        return self.requests[-1]["request_time"]

    def fastest(self) -> float:
        """
        Returns the slowest request's completion time

        >>> results = Results(10.6, [{
        ...     'status_code': 200,
        ...     'request_time': 3.4
        ... }, {
        ...     'status_code': 500,
        ...     'request_time': 6.1
        ... }, {
        ...     'status_code': 200,
        ...     'request_time': 1.04
        ... }])
        >>> results.fastest()
        1.04
        """
        return self.requests[0]["request_time"]

    def average_time(self) -> float:
        """
        Returns the slowest request's completion time

        >>> results = Results(10.6, [{
        ...     'status_code': 200,
        ...     'request_time': 3.4
        ... }, {
        ...     'status_code': 500,
        ...     'request_time': 6.1
        ... }, {
        ...     'status_code': 200,
        ...     'request_time': 1.04
        ... }])
        >>> results.average_time()
        3.513333333333333
        """
        return statistics.mean([r["request_time"] for r in self.requests])

    def successful_requests(self) -> int:
        """
        Returns the slowest request's completion time

        >>> results = Results(10.6, [{
        ...     'status_code': 200,
        ...     'request_time': 3.4
        ... }, {
        ...     'status_code': 500,
        ...     'request_time': 6.1
        ... }, {
        ...     'status_code': 200,
        ...     'request_time': 1.04
        ... }])
        >>> results.successful_requests()
        2
        """
        return len([r for r in self.requests if r["status_code"] in range(200, 299)])

    def rpm(self) -> int:
        """
        Returns the requests per minute

        >>> results = Results(1.5, [{
        ...     'status_code': 200,
        ...     'request_time': 0.25
        ... }, {
        ...     'status_code': 500,
        ...     'request_time': 1.0
        ... }, {
        ...     'status_code': 200,
        ...     'request_time': 0.25
        ... }])
        >>> results.rpm()
        120
        """
        return 60 * round(len(self.requests) / self.total_time)

    def rps(self) -> int:
        """
        Returns the requests per minute

        >>> results = Results(1.5, [{
        ...     'status_code': 200,
        ...     'request_time': 0.25
        ... }, {
        ...     'status_code': 500,
        ...     'request_time': 1.0
        ... }, {
        ...     'status_code': 200,
        ...     'request_time': 0.25
        ... }])
        >>> results.rps()
        2
        """
        return round(self.rpm() / 60)
